#include "bright.h"

/* FUNCTIONS */

bright::bright(int size_x, int size_y, int p_size_x, int p_size_y, int min, int max, 
	       int size_window, double min_corr, char *device) : QDialog() {

  setupUi(this);

  // laser calibration
  connect(btPict1, SIGNAL(clicked()), this, SLOT(takePicture1()));
  connect(btPict2, SIGNAL(clicked()), this, SLOT(takePicture2()));
  connect(btClic, SIGNAL(clicked()), this, SLOT(clicCorners()));
  connect(btColour, SIGNAL(clicked()), this, SLOT(takeColours()));
  connect(btFind, SIGNAL(clicked()), this, SLOT(findCorners()));
  connect(btLaser, SIGNAL(clicked()), this, SLOT(findLaser()));
  connect(btCalib, SIGNAL(clicked()), this, SLOT(calibrateLaser()));

  // platform calibration
  connect(btPict, SIGNAL(clicked()), this, SLOT(takePicturePlatform()));
  connect(btClic2, SIGNAL(clicked()), this, SLOT(clicCornersPlatform()));
  connect(btColour2, SIGNAL(clicked()), this, SLOT(takeColoursPlatform()));
  connect(btFindLeft, SIGNAL(clicked()), this, SLOT(findCornersLeft()));
  connect(btFindRight, SIGNAL(clicked()), this, SLOT(findCornersRight()));
  connect(btCalib2, SIGNAL(clicked()), this, SLOT(calibratePlatform()));

  // 3D reconstruction
  connect(btTakePicture, SIGNAL(clicked()), this, SLOT(takePicture()));
  connect(btGetPoints, SIGNAL(clicked()), this, SLOT(getPoints()));
  connect(btScan, SIGNAL(clicked()), this, SLOT(startScan()));

  // configuration
  this->size_x = size_x;
  this->size_y = size_y;
  this->p_size_x = p_size_x;
  this->p_size_y = p_size_y;
  this->t_min = min;
  this->t_max = max;
  this->size_window = size_window;
  this->min_corr = min_corr;
  this->device = string(device);

  is_points_platform[0] = is_points_platform[1] = false;
  is_picts[0] = is_picts[1] = false;
  is_pict_3d = false;
  is_pict_platform = false;

  current = -1;
  colour = CORNER;

}

/* READ FROM THE CONFIGURATION */

Camera::Matrix bright::getIntrinsics() throw (mimasexception) {

  Camera::Matrix intrinsics(3,3);

  bool ok;

  intrinsics(0,0) = k00->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(0,1) = k01->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(0,2) = k02->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(1,0) = k10->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(1,1) = k11->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(1,2) = k12->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(2,0) = k20->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(2,1) = k21->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  intrinsics(2,2) = k22->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");

  return intrinsics;

}

Camera::Vector bright::getLaserNormal() throw (mimasexception) {

  Camera::Vector normal(3);

  bool ok;

  normal(0) = ln0->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  normal(1) = ln1->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  normal(2) = ln2->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");

  return normal;

}

Camera::Vector bright::getLaserPoint() throw (mimasexception) {

  Camera::Vector point(3);

  bool ok;

  point(0) = lp0->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  point(1) = lp1->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  point(2) = lp2->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");

  return point;

}

Camera::Vector bright::getPlatformVector() throw (mimasexception) {

  Camera::Vector vect(3);

  bool ok;

  vect(0) = pl0->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  vect(1) = pl1->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  vect(2) = pl2->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");

  return vect;

}

Camera::Vector bright::getPlatformPoint() throw (mimasexception) {

  Camera::Vector point(3);

  bool ok;

  point(0) = pp0->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  point(1) = pp1->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");
  point(2) = pp2->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");

  return point;

}

double bright::getStepDegrees() throw (mimasexception) {

  double theta;

  bool ok;

  theta = lineStep->text().toDouble(&ok);
  MMERROR(ok, mimasexception, , "Bad input");

  return theta;

}

/* SLOTS */

////// laser calibration

void bright::takePicture1() {

  try {
    
    if(is_picts[0])
      delete picts[0];

    picts[0] = new Picture(device);
    lbPict->setImage(picts[0]);

    current = 0;
    is_picts[0] = true;

  } catch(mimasexception &e) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Couldn't read the video device."));
    
  }

}

void bright::takePicture2() {
  
  try {

    if(is_picts[1])
      delete picts[1];

    picts[1] = new Picture(device);
    lbPict->setImage(picts[1]);

    current = 1;
    is_picts[1] = true;

  } catch(mimasexception &e) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Couldn't read the video device."));

  }

}

void bright::clicCorners() {

  lbPict->resetCorners();

}


void bright::takeColours() {

  if(current == -1) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Take a picture first."));
    return;

  } else if (colour == CORNER) {

    colour = BLACK;
    lbNotes->setText("Click on a black square of the chessboard.");
    lbPict->click(colour);

  } else if (colour == BLACK) {

    colour = WHITE;
    lbNotes->setText("Click on a white area of the chessboard.");
    lbPict->click(colour);

  } else {

    colour = CORNER;
    lbNotes->setText("OK");
    lbPict->click(colour);

  }

}


void bright::findCorners() {

  lbNotes->setText("Click on the four extreme corners.");

  try {

    lbPict->findCorners(size_window, min_corr);
    lbNotes->setText("Corners found.");

  } catch (mimasexception &e) {

    lbNotes->setText("Corners not found.");
  
  }

}

void bright::findLaser() {

  if(current == -1) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"), QString("Take a picture first."));
    return;

  } else {

    // define some colours
    pixel_rgba black((unsigned char)0, (unsigned char)0, (unsigned char)0, 0);
    // pixel_rgba white((unsigned char)255, (unsigned char)255, (unsigned char)255, 0);
    // pixel_rgba min((unsigned char)t_min, (unsigned char)t_min, (unsigned char)t_min, 0);
    // pixel_rgba max((unsigned char)t_max, (unsigned char)t_max, (unsigned char)t_max, 0);

    image< rgba< unsigned char > > img_color( picts[current]->getImage() );

    // apply the threshold
    image< unsigned char > gray_img( img_color );
    image< unsigned char > img( bilevel_double< unsigned char >( gray_img, t_min, t_max, 0, 255 ) );

    // 0.99 to try to get just one line (must be < 1)
    find_line< image< unsigned char >::iterator > fl(img.ul(), img.lr(), 0.99);

    // get the lines from the Hough transform.
    vector< segment< point > > lines = fl.getLines();

#ifndef NDEBUG
    std::cout << "Number of lines found : " << lines.size() << std::endl;
#endif
  
    if(lines.size() > 0) {
    
      // draw only the first line. Hopefully there should be only one in the vector!
      // even if there are more line, we consider only the first one of the vector.
      drawLine(img_color, lines.at(0).p1.x, lines.at(0).p1.y, lines.at(0).p2.x, lines.at(0).p2.y, black);
#ifndef NDEBUG
      std::cout << lines.at(0).p1.x << ", " << lines.at(0).p1.y << " /  " 
		<< lines.at(0).p2.x << ", " << lines.at(0).p2.y << std::endl;
#endif

      // save the laser line
      laser_lines[current] = new segment< point >(lines.at(0));

      // remove the old picture
      delete picts[current];

      // print the new picture with the laser detected
      picts[current] = new Picture(img_color);
      lbPict->setImage(picts[current]);

      lbNotes->setText("Laser line found.");

    } else {

      lbNotes->setText("Laser line not found.");

    }

  }

}

void bright::calibrateLaser() {

  if(!is_picts[0] || !is_picts[1]) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"), QString("Take 2 pictures first."));
    return;

  } else {

    std::vector< Chessboard > chess;
    std::vector< Camera::Matrix > homographies, extrinsics;  
    std::vector< Camera::Vector > points;

    // for the 2 picture, save the corners and create the chessboard objects
    for(int i = 0; i < 2; ++i) {

      Chessboard c;

      c.image.push_back(pixel_f(picts[i]->corns[0].x, picts[i]->corns[0].y));
      c.image.push_back(pixel_f(picts[i]->corns[1].x, picts[i]->corns[1].y));
      c.image.push_back(pixel_f(picts[i]->corns[2].x, picts[i]->corns[2].y));
      c.image.push_back(pixel_f(picts[i]->corns[3].x, picts[i]->corns[3].y));

      c.object.push_back(point_f(0,              0));
      c.object.push_back(point_f(2 * size_x - 1, 0));
      c.object.push_back(point_f(2 * size_x - 1, 2 * size_y - 1));
      c.object.push_back(point_f(0,              2 * size_y - 1));

      chess.push_back(c);

      homographies.push_back(Camera::Matrix(3,3));
      extrinsics.push_back(Camera::Matrix(3,4));

    }
    
    try {
  
      // find all the homographies between the chessboard and the images
      Camera::findHomographies(homographies, chess);

      // get the intrinsic matrix from the user configuration
      Camera::Matrix intrinsics = getIntrinsics();

      // find the extrinsics parameters
      Camera::findExtrinsics(extrinsics, homographies, intrinsics, chess.size());

      // convert the laser line into the camera scale
      for(int i = 0; i < 2; ++i) {
	
	// convert from the image frame to the chessboard frame using H^{-1}
	Camera::Matrix h_inv = inv< double >(homographies[i]);
	Camera::Vector v1(3), v2(3);

	v1(0) = laser_lines[i]->p1.x;
	v1(1) = laser_lines[i]->p1.y;
	v1(2) = 1;

	v2(0) = laser_lines[i]->p2.x;
	v2(1) = laser_lines[i]->p2.y;
	v2(2) = 1;

	v1 = boost::numeric::ublas::prod(h_inv, v1);
	v2 = boost::numeric::ublas::prod(h_inv, v2);

	// normalize the vector
	for( int k = 0; k < 3; ++k ) {
	  v1(k) /= v1(2);
	  v2(k) /= v2(2);
	}

	Camera::Vector u1(4), u2(4);
	Camera::Matrix Q(4, 4);

	u1(0) = v1(0);
	u1(1) = v1(1);
	u1(2) = 0;
	u1(3) = 1;

	u2(0) = v2(0);
	u2(1) = v2(1);
	u2(2) = 0;
	u2(3) = 1;

	// use the extrinsic parameters to convert from the chessboard frame
	// to the camera frame (ie world reference frame)
	for(int ii=0; ii<3; ++ii)
	  for(int jj=0; jj<4; ++jj)
	    Q(ii, jj) = extrinsics[i](ii, jj);

	Q(3,0) = 0;
	Q(3,1) = 0;
	Q(3,2) = 0;
	Q(3,3) = 1;

	u1 = boost::numeric::ublas::prod(Q, u1);
	u2 = boost::numeric::ublas::prod(Q, u2);

	// normalize
	for(int k = 0; k < 4; ++k){
	  u1(k) /= u1(3);
	  u2(k) /= u2(3);
	}

	// save the result : 2 points of the laser line in the camera frame
	points.push_back(u1);
	points.push_back(u2);

      }

      // compute 2 vectors belonging to the plane
      Camera::Vector u(3), v(3), n;

      u(0) = points[1](0) - points[0](0);
      u(1) = points[1](1) - points[0](1);
      u(2) = points[1](2) - points[0](2);
      
      v(0) = points[3](0) - points[2](0);
      v(1) = points[3](1) - points[2](1);
      v(2) = points[3](2) - points[2](2);

      // the cross product is a normal vector of the plane
      n = crossProduct(u, v);

      // display the result
      std::cout << "Laser calibration result :" << std::endl;
      std::cout << "Normal vector :" << std::endl;
      for( int k = 0; k < 3; ++k ) {
	std::cout << n(k) << " ";
      }
      std::cout << std::endl;
      std::cout << std::endl;

      // take an arbitrary point belonging to the plane (the first one for instance)
      std::cout << "Point :" << std::endl;
      for( int k = 0; k < 3; ++k ) {
	std::cout << points[0](k) << " ";
      }
      std::cout << std::endl;
      std::cout << std::endl;

      // save in the configuration section
      ln0->setText(QString("%1").arg(n(0)));
      ln1->setText(QString("%1").arg(n(1)));
      ln2->setText(QString("%1").arg(n(2)));

      lp0->setText(QString("%1").arg(points[0](0)));
      lp1->setText(QString("%1").arg(points[0](1)));
      lp2->setText(QString("%1").arg(points[0](2)));

      lbNotes->setText("Laser calibration computed");

    } catch (mimasexception &e) {

      QMessageBox::critical(0, QString("3D Laser reconstruction"),
			    QString("Could not compute the laser calibration\n")
			    + QString(e.what()));

    }

  }

}



////// platform calibration

void bright::takePicturePlatform() {

  try {
    
    if(is_pict_platform)
      delete pict_platform;

    pict_platform = new Picture(device);
    lbPict3->setImage(pict_platform);

    is_pict_platform = true;

  } catch(mimasexception &e) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Couldn't read the video device."));
    
  }

}


void bright::clicCornersPlatform() {

  lbPict3->resetCorners();

}

void bright::takeColoursPlatform() {

  if(!is_pict_platform) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Take a picture first."));
    return;

  } else if (colour == CORNER) {

    colour = BLACK;
    lbNotes->setText("Click on a black square of the chessboard.");
    lbPict3->click(colour);

  } else if (colour == BLACK) {

    colour = WHITE;
    lbNotes->setText("Click on a white area of the chessboard.");
    lbPict3->click(colour);

  } else {

    colour = CORNER;
    lbNotes->setText("OK");
    lbPict3->click(colour);

  }

}

// left and right are actually arbitrary chosen
void bright::findCornersLeft() {

  lbNotes->setText("Click on the 4 extreme corners of the left chessboard.");

  try {

    points_platform[0] = lbPict3->findCornersPlatform(size_window, min_corr, p_size_x, p_size_y);
    lbNotes->setText("Corners found.");
    is_points_platform[0] = true;

  } catch (mimasexception &e) {

    lbNotes->setText("Corners not found.");
  
  }

}

void bright::findCornersRight() {

  lbNotes->setText("Click on the 4 extreme corners of the right chessboard.");

  try {

    points_platform[1] = lbPict3->findCornersPlatform(size_window, min_corr, p_size_x, p_size_y);
    lbNotes->setText("Corners found.");
    is_points_platform[1] = true;

  } catch (mimasexception &e) {

    lbNotes->setText("Corners not found.");
  
  }

}

void bright::calibratePlatform() {

  if(!is_points_platform[0] || !is_points_platform[1]) {
      QMessageBox::critical(0, QString("3D reconstruction"),
			    QString("Please get the left and right chessboards before."));
  } else {

    std::vector< Chessboard > chess;
    std::vector< Camera::Matrix > homographies, extrinsics;  
    std::vector< Camera::Vector > points;

    // there is only one picture but 2 chessboards, so we put both of them in the vector
    for(int i = 0; i < 2; ++i) {

      Chessboard c;

      c.image.push_back(points_platform[i][0]);    
      c.image.push_back(points_platform[i][1]);
      c.image.push_back(points_platform[i][2]);
      c.image.push_back(points_platform[i][3]);

      c.object.push_back(point_f(0,                0));
      c.object.push_back(point_f(2 * p_size_x - 1, 0));
      c.object.push_back(point_f(2 * p_size_x - 1, 2 * p_size_y - 1));
      c.object.push_back(point_f(0,                2 * p_size_y - 1));

      chess.push_back(c);

      homographies.push_back(Camera::Matrix(3,3));
      extrinsics.push_back(Camera::Matrix(3,4));

    }

    try {
  
      // find all the homographies between the chessboard and the images
      Camera::findHomographies(homographies, chess);

      // get the intrinsic matrix from the user configuration
      Camera::Matrix intrinsics = getIntrinsics();

      // find the extrinsics parameters
      Camera::findExtrinsics(extrinsics, homographies, intrinsics, chess.size());

      for(int i = 0; i < 2; ++i) {

	// convert from the chessboard frame to the camera frame
	Camera::Vector u1(4), u2(4), u3(4);
	Camera::Matrix Q(4, 4);

	u1(0) = 0;
	u1(1) = 0;
	u1(2) = 0;
	u1(3) = 1;

	u2(0) = 2 * p_size_x - 1;
	u2(1) = 0;
	u2(2) = 0;
	u2(3) = 1;

	u3(0) = 2 * p_size_x - 1;
	u3(1) = 2 * p_size_y - 1;
	u3(2) = 0;
	u3(3) = 1;

	for(int ii=0; ii<3; ++ii)
	  for(int jj=0; jj<4; ++jj)
	    Q(ii, jj) = extrinsics[i](ii, jj);

	Q(3,0) = 0;
	Q(3,1) = 0;
	Q(3,2) = 0;
	Q(3,3) = 1;

	u1 = boost::numeric::ublas::prod(Q, u1);
	u2 = boost::numeric::ublas::prod(Q, u2);
	u3 = boost::numeric::ublas::prod(Q, u3);

	// normalize
	for(int k = 0; k < 4; ++k){
	  u1(k) /= u1(3);
	  u2(k) /= u2(3);
	  u3(k) /= u3(3);
	}

	Camera::Vector u(3), v(3), n;

	// save a point of the plane in the camera frame
	points.push_back(u1);

	// compute 2 vectors of the plane
	u(0) = u2(0) - u1(0);
	u(1) = u2(1) - u1(1);
	u(2) = u2(2) - u1(2);
      
	v(0) = u3(0) - u1(0);
	v(1) = u3(1) - u1(1);
	v(2) = u3(2) - u1(2);

	// the cross product is a normal vector of the plane
	n = crossProduct(u, v);

	// save a normal vector of the plane in the camera frame
	points.push_back(n);

      }

      // the intersection of 2 planes is a line, defined by a point and a vector
      Camera::Vector n_line, p_line(3);

      // the vector is normal to the 2 planes normal vectors.
      n_line = crossProduct(points[1], points[3]);
      
      // normalize the vector
      double norm = sqrt(n_line(0) * n_line(0) + n_line(1) * n_line(1) + n_line(2) * n_line(2));
      n_line(0) /= norm;
      n_line(1) /= norm;
      n_line(2) /= norm;

      // see the report for the formula of the intersection of 2 planes
      p_line(0) = 
	( points[3](2) * ( points[1](0) * points[0](0) + points[1](2) * points[0](2) )
	  - points[1](2) * ( points[3](0) * points[2](0) + points[3](2) * points[2](2) ) )
	/ ( points[1](0) * points[3](2) - points[1](2) * points[3](0) );
      p_line(1) = 0;
      p_line(2) = 
	( points[3](0) * ( points[1](0) * points[0](0) + points[1](2) * points[0](2) )
	  - points[1](0) * ( points[3](0) * points[2](0) + points[3](2) * points[2](2) ) )
	/ ( points[1](2) * points[3](0) - points[1](0) * points[3](2));

      // display
      std::cout << "Platform calibration result :" << std::endl;
      std::cout << "Line vector :" << std::endl;
      for( int k = 0; k < 3; ++k ) {
	std::cout << n_line(k) << " ";
      }
      std::cout << std::endl;
      std::cout << std::endl;

      std::cout << "Point :" << std::endl;
      for( int k = 0; k < 3; ++k ) {
	std::cout << p_line(k) << " ";
      }
      std::cout << std::endl;
      std::cout << std::endl;

      // save in the configuration section
      pp0->setText(QString("%1").arg(p_line(0)));
      pp1->setText(QString("%1").arg(p_line(1)));
      pp2->setText(QString("%1").arg(p_line(2)));

      pl0->setText(QString("%1").arg(n_line(0)));
      pl1->setText(QString("%1").arg(n_line(1)));
      pl2->setText(QString("%1").arg(n_line(2)));

      lbNotes->setText("Platform calibration computed");

    } catch (mimasexception &e) {

      QMessageBox::critical(0, QString("3D Laser reconstruction"),
			    QString("Could not compute the laser calibration\n")
			    + QString(e.what()));

    }

  }

}


////// 3D reconstruction

void bright::takePicture() {

  try {
    
    if(is_pict_3d)
      delete pict_3d;

    pict_3d = new Picture(device);
    lbPict2->setImage(pict_3d);

    is_pict_3d = true;

  } catch(mimasexception &e) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Couldn't read the video device."));
    
  }

}

int filter_double(int n) {

  double x = (double)n / 256;
  x *= x;
  return (int)(x * 256);

}

// find the best point on a line.
// really need to be improved !
int findMaximum(boost::detail::multi_array::const_sub_array<unsigned char, 1u> tab, int size) {

  int max = -1;
  std::vector< int > vect;

  for(int i=0; i<size; ++i)
    if(tab[i] != 0) 
      vect.push_back(i);

  // compute the mean (method too easy ! find another one)
  if(vect.size() > 0)
    max = accumulate(vect.begin(), vect.end(), 0) / vect.size();

  return max;

}

// this function is only made for testing purposes, it can only reconstruct
// the points for a simple laser line
void bright::getPoints() {

  if(!is_pict_3d) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Take a picture first."));

  } else {

    image< rgba< unsigned char > > img = pict_3d->getImage();
    image<unsigned char> gray_image = image<unsigned char>(img);

    // threshold
    gray_image = bilevel_double< unsigned char >(gray_image, 200, 255, 0, 255);

    delete pict_3d;
    pict_3d = new Picture(gray_image);
    lbPict2->setImage(pict_3d);

    boost::const_multi_array_ref< unsigned char, 2 > data
      ( gray_image.rawData(),
        boost::extents[ gray_image.getHeight() ][ gray_image.getWidth() ] );
    
    std::vector< point > points;

    for(int i=0; i<gray_image.getHeight(); ++i) {

      int max = findMaximum(data[i], gray_image.getWidth());

      if(max != -1) {
	points.push_back(point(max, i));
	std::cout << "Point : " << max << ", " << i << std::endl;
      }
      
    }
    
    try {

      compute3DpointsLine(points, getIntrinsics(), getLaserNormal(), getLaserPoint());

    } catch(mimasexception &e) {

    QMessageBox::critical(0, QString("3D Laser reconstruction"),
			  QString("Bad input."));
    
    }


  }

}


void bright::startScan() {

  int angle = 0;
  double theta = 0;

  platform platform;

  // open the output file
  ofstream f(OUTPUT);
  if(f.is_open())
    cout << "Points written in " << OUTPUT << endl;
  else {
    lbNotes->setText("Could not open the output file");
    return;
  }

  // use degrees to avoid round errors
  while(angle < 360) {

    try {
    
      if(is_pict_3d)
	delete pict_3d;

      // take a picture
      pict_3d = new Picture(device);
      lbPict2->setImage(pict_3d);
      is_pict_3d = true;

    } catch(mimasexception &e) {

      QMessageBox::critical(0, QString("3D Laser reconstruction"),
			    QString("Couldn't read the video device."));
    
    }

    image< rgba< unsigned char > > img = pict_3d->getImage();
    image<unsigned char> gray_image = image<unsigned char>(img);

    // threshold
    gray_image = bilevel_double< unsigned char >(gray_image, 200, 255, 0, 255);

    // find the 2-D points of the laser line on the object
    boost::const_multi_array_ref< unsigned char, 2 > data
      ( gray_image.rawData(),
        boost::extents[ gray_image.getHeight() ][ gray_image.getWidth() ] );
    std::vector< point > points;

    // loop on the lines of the image
    for(int i=0; i<gray_image.getHeight(); ++i) {

      int max = findMaximum(data[i], gray_image.getWidth());
      if(max != -1)
	points.push_back(point(max, i));

    }
    
    try {

      // compute the 3-D points from the 2-D points
      compute3Dpoints(points, getIntrinsics(), getLaserNormal(), getLaserPoint(), 
		      getPlatformVector(), getPlatformPoint(), theta, f);

    } catch(mimasexception &e) {

      QMessageBox::critical(0, QString("3D Laser reconstruction"),
			    QString("Bad input."));
    
    }

    // rotate the platform
    platform.rotatePlatform(getStepDegrees());

    // change the theta
    angle += (int) getStepDegrees();
    theta += getStepDegrees() * PI / 180;

  }

  f.close();
  lbNotes->setText("3-D reconstruction done.");

}
