#include "displaypicture.h"

DisplayPicture::DisplayPicture(QWidget * parent, Qt::WFlags f):QLabel(parent, f){

  isPict = false;

}

void 
DisplayPicture::setImage(Picture *p) {

  pict = p;

  if( !isPict )
    isPict = true;

  setPixmap(pict->getPixmap());

}

void 
DisplayPicture::findCorners(int size_window, double min_corr) throw (mimasexception) {

  if(isPict) {

    if(pict->points.size() != 4) {
      QMessageBox::critical(0, QString("3D reconstruction"),
			    QString("Need to click on the four extrem corners."));
      MMERROR(0, mimasexception, , "Corners");
    }

    // test black and white TODO

    corner_detection c(1, 1, size_window, white_pixel, black_pixel);

    try {

      c.findCorners(pict->getImage(), pict->points, min_corr);

      corner_detection::MatrixPixel corners = c.getCorners();

      pict->points.clear();

      for (unsigned i = 0; i < corners.size1 (); ++ i)
	for (unsigned j = 0; j < corners.size2 (); ++ j)
	  pict->points.push_back(pixel_f(corners(i, j).x, corners(i, j).y));

      pict->corns[0] = corners(0, 0);
      pict->corns[1] = corners(0, 1);
      pict->corns[2] = corners(1, 1);
      pict->corns[3] = corners(1, 0);

      update();

    } catch (mimasexception &e) {

      QMessageBox::critical(0, QString("3D reconstruction"),
			    QString("Corners not found. Try to snap a better picture.\n Cause : ")
			    += QString(e.what()));
      throw e;

    }

  }

}

std::vector< pixel_f >
DisplayPicture::findCornersPlatform(int size_window, double min_corr, int p_size_x, int p_size_y)
     throw (mimasexception) {

  std::vector< pixel_f > corns;

  if(isPict) {

    if(pict->points.size() < 4) {
      QMessageBox::critical(0, QString("3D reconstruction"),
			    QString("Need to click on the four extrem corners."));
      MMERROR(0, mimasexception, , "Corners");
    }

    corner_detection c(p_size_x, p_size_y, size_window, white_pixel, black_pixel);

    try {

      vector< pixel_f > points;

      for(int k=0; k<4; ++k) {
	points.push_back(pict->points.back());
	pict->points.pop_back();
      }

      reverse(points.begin(), points.end());

      c.findCorners(pict->getImage(), points, min_corr);

      corner_detection::MatrixPixel corners = c.getCorners();

      for (unsigned i = 0; i < corners.size1 (); ++ i)
	for (unsigned j = 0; j < corners.size2 (); ++ j)
	  pict->points.push_back(pixel_f(corners(i, j).x, corners(i, j).y));

      corns.push_back(corners(0,            0));
      corns.push_back(corners(0,            2*p_size_x-1));
      corns.push_back(corners(2*p_size_y-1, 2*p_size_x-1));
      corns.push_back(corners(2*p_size_y-1, 0));

      update();

    } catch (mimasexception &e) {

      QMessageBox::critical(0, QString("3D reconstruction"),
			    QString("Corners not found. Try to snap a better picture.\n Cause : ")
			    += QString(e.what()));
      throw e;

    }

  }

  return corns;

}


void 
DisplayPicture::mousePressEvent(QMouseEvent *event) {

  if(event->button() == Qt::LeftButton && isPict) {

    if(action == BLACK) {

      pixel_rgba black = pict->getImage().getPixel(event->x(), event->y());
      black_pixel = black;

    } else if(action == WHITE) {

      pixel_rgba white = pict->getImage().getPixel(event->x(), event->y());
      white_pixel = white;

    } else {

      // add the point to the list
      pict->points.push_back(pixel_f(event->x(), event->y()));

      // redraw the picture
      update();

    } 

  }
  
}

void
DisplayPicture::resetCorners() {

  if(isPict){

    pict->points.clear();
    update();

  }

}

static inline void drawCross(QPainter &paint, int x, int y) {

  paint.drawLine(x, y-2, x, y+2);
  paint.drawLine(x-2, y, x+2, y);

}

static inline int approx(double x) {

  return (int)(x + 0.5);

}

void 
DisplayPicture::paintEvent(QPaintEvent */*event*/) {

  QPainter paint(this);
  paint.setPen(Qt::red);

  if(isPict) {

    // draw the picture
    paint.drawPixmap(0,0, pict->getPixmap());
    
    // draw all the points
    for( std::vector< pixel_f >::iterator it = pict->points.begin(); it != pict->points.end(); ++it ) {
      drawCross(paint, approx(it->x), approx(it->y));
    }

  }

}

void 
DisplayPicture::click(int n) {

  action = n;

}
