#include "points.h"

void compute3DpointsLine(vector< point > points, Camera::Matrix intrinsics, 
			 Camera::Vector plane_normal, Camera::Vector plane_point) {

  Camera::Vector pixel_point(3), line;
  pixel_point(2) = 1;

  ofstream f(OUTPUT);
  if(f.is_open())
    cout << "Points written in " << OUTPUT << endl;
  else
    return;

  for(int i = 0; i < (int) points.size(); ++i) {

    pixel_point(0) = points[i].x;
    pixel_point(1) = points[i].y;

    // get the vector of the line passing through the pixel
    line = boost::numeric::ublas::prod(inv(intrinsics), pixel_point);

    double k = 
      (plane_normal(0) * plane_point(0) + plane_normal(1) * plane_point(1) + plane_normal(2) * plane_point(2)) / 
      (plane_normal(0) * line(0)        + plane_normal(1) * line(1)        + plane_normal(2) * line(2));

    // save in a file
    if (f.is_open()) {
      f << k * line(0) << " " 
	<< - k * line(1) << " " 
	<< k * line(2) << endl;
    }
    
  }
  
  f.close();

}

void compute3Dpoints(vector< point > points, Camera::Matrix intrinsics, 
		     Camera::Vector plane_normal, Camera::Vector plane_point, 
		     Camera::Vector platform_vector, Camera::Vector platform_point,
		     double theta, ofstream &file) {

  Camera::Vector pixel_point(3), line;
  pixel_point(2) = 1;

  for(int i = 0; i < (int) points.size(); ++i) {

    pixel_point(0) = points[i].x;
    pixel_point(1) = points[i].y;

    // get the vector of the line passing through the pixel
    line = boost::numeric::ublas::prod(inv(intrinsics), pixel_point);

    double k = 
      (plane_normal(0) * plane_point(0) + plane_normal(1) * plane_point(1) + plane_normal(2) * plane_point(2)) / 
      (plane_normal(0) * line(0)        + plane_normal(1) * line(1)        + plane_normal(2) * line(2));

    // get the 3-D point
    line *= k;

    // rotate it
    line -= platform_point;
    line = rodrigues< double >(platform_vector, line, theta);
    line += platform_point;

    // save in a file
    file << line(0) << " " 
	 << line(1) << " " 
	 << line(2) << endl;
    
  }

}
